/*
  defines layout of the main window (buttons, fields, menus and so on)
  creates actions
  creates the signal slot mechanism
  provides printing
  initiates simulation of economic growth
*/
template <class X> X Toggle(X x) { return (x==false) ? true : false ;}
#include "00FrontEnd.h"

#include <QtWidgets/QMenu>           
#include <QtWidgets/QMenuBar>        
#include <QtWidgets/QApplication>
#include <QtGui/QIcon>
#include <QtWidgets/QMessageBox> 
#include <QtWidgets/QWhatsThis>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton> 
#include <QtWidgets/QToolTip>
#include <QtCore/QSize>
#include <QtPrintSupport/QPrinter>
#include <QtPrintSupport/QPrintDialog>

#include "LCD.h"
#include "LMMarket.h"
#include "ISMarket.h"
#include "IS-LM.h"
#include "Equilibrium.h"
#include "WorldMarket.h"

const int voffset = 70;

/**********************************************************************/

HBWidget::HBWidget(QWidget *parent)
    : QMainWindow(parent)
{
  WindowWidth = 800;
  WindowHeight = 600;
  SCALE = 0.75 ;
  hskip = int(SCALE * 10) ;
#if __GNUG__ /* GNU C++ Compiler Linux */
  setFont( QFont( "helvetica", 10, 25 ) );
#else        /* C++ for Mickysoft Windows */
  setFont( QFont( "helvetica", 8, 25 ) );
#endif 
  setWindowTitle( tr("TEXT headline") );
  printer = new QPrinter;

  createActions();
  createMenus();
  setIconSize(QSize(24, 24));
  createToolBars();
  createStatusBar();

  /** alle verwendeten Element definieren ********************************/

  V__LCD = new LCDRange( "k", this );   // V__ denotes k (V=1/k) !
  V__LCD->setRange( 0, 10000 );
  M_LCD  = new LCDRange( "M", this );
  M_LCD->setRange( 0, 10000 );
  c_LCD = new LCDRange( "c", this );
  c_LCD->setRange( 0, 10000 );
  t_LCD = new LCDRange( "tax", this );
  t_LCD->setRange( 0, 10000 );
  C0_LCD = new LCDRange( "Caut", this );
  C0_LCD->setRange( 0, 10000 );
  G_LCD = new LCDRange( "G", this );
  G_LCD->setRange( 0, 10000 );
  w_LCD = new LCDRange( "w_min", this );
  w_LCD->setRange( 0, 10000 );
  P_LCD = new LCDRange( "P", this );
  P_LCD->setRange( 0, 10000 );
  e_LCD = new LCDRange( "e", this );
  e_LCD->setRange( 0, 10000 );
  K_LCD = new LCDRange( "K", this );
  K_LCD->setRange( 0, 10000 );
  alpha_LCD = new LCDRange( "alpha", this );
  alpha_LCD->setRange( 0, 10000 );
  LMMarket = new MoneyMarket( this );
  ISMarket = new CommodityMarket( this );
  ISLMMarket = new ISLMField( this );
  aggrMarket = new Equilibrium( this );

  /** Positionierung der Elemente, Reihenfolge wichtig ******************/

  rb = new QGroupBox(tr("Keynesian Cases"), this );
  keynes1 = new QRadioButton(tr("Liquidity trap"));
  keynes2 = new QRadioButton(tr("Investment trap"));
  keynes3 = new QRadioButton(tr("Wage rate rigidities"));
  keynes4 = new QRadioButton(tr("Standard case"));
  keynes4->setChecked(true);
  QVBoxLayout *vbox = new QVBoxLayout();
  vbox->addWidget(keynes1);
  vbox->addWidget(keynes2);
  vbox->addWidget(keynes3);
  vbox->addWidget(keynes4);
  keynes1->setToolTip(tr("TOOLTIP Keynes 1"));
  keynes2->setToolTip(tr("TOOLTIP Keynes 2"));
  keynes3->setToolTip(tr("TOOLTIP Keynes 3"));
  keynes4->setToolTip(tr("TOOLTIP Keynes 4"));
  vbox->addStretch(1);
  rb->setLayout(vbox);

  WidgetPositions() ;

  /** signals und slots zuordnen ****************************************/

  connect( keynes1, SIGNAL(clicked()), this, SLOT(KeynesMessage()) );
  connect( keynes2, SIGNAL(clicked()), this, SLOT(KeynesMessage()) );
  connect( keynes3, SIGNAL(clicked()), this, SLOT(KeynesMessage()) );
  connect( keynes4, SIGNAL(clicked()), this, SLOT(KeynesMessage()) );
  connect( aggrMarket, SIGNAL(wminClicked()), SLOT(wminClicked()) );
  connect( ISLMMarket, SIGNAL(AdjustPriceLevel( int )), 
           aggrMarket, SLOT(Adjustment( int )));
  connect( ISMarket, SIGNAL( explain( const QString& ) ), 
	   this, SLOT(updateStatusBar( const QString& )));
  connect( LMMarket, SIGNAL( explain( const QString& ) ), 
	   this, SLOT(updateStatusBar( const QString& )));
  connect( ISLMMarket, SIGNAL( explain( const QString& ) ), 
	   this, SLOT(updateStatusBar( const QString& )));
  connect( ISLMMarket, SIGNAL( updateProgressBar( int ) ), 
	   this, SLOT( updateProgress( int )));
  connect( aggrMarket, SIGNAL( explain( const QString& ) ), 
	   this, SLOT(updateStatusBar( const QString& )));
  connect( aggrMarket, SIGNAL( updateProgressBar( int ) ), 
	   this, SLOT( updateProgress( int )));

  connect( ISMarket,   SIGNAL(ISMarket2ISLM(double,double,double,double,
					    bool, double, double)), 
	   ISLMMarket, SLOT(ISChanged(double,double,double,double,
				      bool, double, double)));
  connect( ISMarket,   SIGNAL( ZCurve2ISLM( double *, double *) ),
	   ISLMMarket, SLOT( Z0CurveChanged( double *, double *)));
  connect( LMMarket, 
	   SIGNAL(LMMarket2ISLM(double,double,double,double,double,double)), 
	   ISLMMarket, SLOT(LMChanged(double,double,double,double,double,double)));
  connect( ISLMMarket, SIGNAL(NewEquilibrium( double *, bool )), 
	   aggrMarket, SLOT(UpdateEquilibrium( double *, bool)));
  connect( ISLMMarket, SIGNAL(YDEqChanged(double)), 
	   SLOT(ISLMChanged(double)));

  connect( ISLMMarket, SIGNAL(EqValuesChanged(double, double)), 
	   LMMarket,   SLOT(EqValuesChanged(double, double)));
  connect( ISLMMarket, SIGNAL(EqValuesChanged(double, double)), 
	   ISMarket,   SLOT(EqValuesChanged(double, double)));
  connect( aggrMarket, SIGNAL(IautChanged(double, double, double )), 
	   ISMarket,   SLOT(IautChanged(double, double, double)));

  connect( ISLMMarket, SIGNAL(MChanged(int)), LMMarket, SLOT(set_M(int)));
  connect( ISLMMarket, SIGNAL(eChanged(int)), ISLMMarket, SLOT(set_e(int)));
  connect( aggrMarket, SIGNAL(MChanged(int)), LMMarket, SLOT(set_M(int)));
  connect( aggrMarket, SIGNAL(GChanged(int)), ISMarket, SLOT(set_G(int)));
  connect( aggrMarket, SIGNAL(CautChanged(int)), ISMarket, SLOT(set_C0(int)));
  connect( V__LCD, SIGNAL(NewVal(int)), LMMarket, SLOT(set_k(int)));
  connect( M_LCD,  SIGNAL(NewVal(int)), LMMarket, SLOT(set_M(int)));
  connect( c_LCD,  SIGNAL(NewVal(int)), ISMarket, SLOT(set_c(int)));
  connect( t_LCD,  SIGNAL(NewVal(int)), ISMarket, SLOT(set_t(int)));

  connect( G_LCD,  SIGNAL(NewVal(int)), ISMarket, SLOT(set_G(int)));
  connect( K_LCD,  SIGNAL(NewVal(int)), aggrMarket, SLOT(set_K(int)));
  connect( P_LCD,  SIGNAL(NewVal(int)), aggrMarket, SLOT(set_P(int)));
  connect( w_LCD,  SIGNAL(NewVal(int)), aggrMarket, SLOT(set_w(int)));
  connect( alpha_LCD, SIGNAL(NewVal(int)), aggrMarket, SLOT(set_alpha(int)));
  connect( e_LCD,  SIGNAL(NewVal(int)), ISLMMarket, SLOT(set_e(int)));

  connect( ISMarket, SIGNAL(cChanged(int)), c_LCD,  SLOT(setValue(int)));
  connect( ISMarket, SIGNAL(tChanged(int)), t_LCD,  SLOT(setValue(int)));
  connect( LMMarket, SIGNAL(kChanged(int)), V__LCD, SLOT(setValue(int)));
  connect( LMMarket, SIGNAL(MChanged(int)), M_LCD,  SLOT(setGValue(int)));

  connect( ISMarket,   SIGNAL(C0Changed(int)),C0_LCD,   SLOT(setGValue(int)));
  connect( C0_LCD,     SIGNAL(NewVal(int)),   ISMarket, SLOT(set_C0(int)));
  connect( ISMarket,   SIGNAL(GChanged(int)), G_LCD, SLOT(setGValue(int)));
  connect( aggrMarket, SIGNAL(KChanged(int)), K_LCD, SLOT(setVValue(int)));
  connect( aggrMarket, SIGNAL(PChanged(int)), P_LCD, SLOT(setPValue(int)));
  connect( aggrMarket, SIGNAL(wChanged(int)), w_LCD, SLOT(setPValue(int)));
  connect( aggrMarket, SIGNAL(alphaChanged(int)), 
	   alpha_LCD,  SLOT(setValue(int)));
  connect( ISLMMarket, SIGNAL(eChanged(int)), e_LCD, SLOT(setPValue(int)));
  connect( ISLMMarket, SIGNAL(eChanged(int)), SLOT(ISMarketChanged()) );
  connect( aggrMarket, SIGNAL(PChanged(int)), SLOT(LMMarketChanged()) );
  connect( aggrMarket, SIGNAL(PChanged(int)), SLOT(ISMarketChanged()) );
  connect(adjustPAction, SIGNAL(triggered()),
	  aggrMarket, SLOT(AdjustPriceLevel())); // see Equilibrium.cpp
  connect(adjustExAction, SIGNAL(triggered()),
	  ISMarket, SLOT(AdjustmentEx()));       // see ISMarket.cpp
  connect(adjustMPAction, SIGNAL(triggered()),
	  ISLMMarket, SLOT(AdjustmentMP()));     // see IS-LM.cpp
  connect(adjusteAction, SIGNAL(triggered()),
	  ISLMMarket,SLOT(AdjustmentERate()));   // see IS-LM.cpp

  w1 = new NewWindow( );
  w1->setGeometry( 10, 10, 500, 200 );
  w1->setWindowTitle( tr("World import market") );
  w2 = new NewWindow( );
  w2->setGeometry( 100, 100, 500, 200 );
  w2->setWindowTitle( tr("World export market") );

  Reset();
  LMMarketChanged();          // --> IS-LM
  ISMarketChanged();          // --> IS-LM
  aggrMarketChanged();
  TooltipsOnOff();
}
void HBWidget::createActions()
{
  /*
    setStatusTip zeigt Text im statusBar (Fuende)
    setToolTip ersetzt "action text" in QAction
   */
  resetAction = new QAction(tr("&Reset Values"), this);
  resetAction->setIcon(QIcon("icons/view-refresh.png"));
  resetAction->setShortcut(tr("Ctrl+R"));
  resetAction->setStatusTip(tr("Reset values to start from scratch."));
  resetAction->setToolTip(tr("Reset values to start from scratch."));
  resetAction->setWhatsThis(tr("Reset values to start from scratch."));
  connect(resetAction,SIGNAL(triggered()),this,SLOT(Reset()));

  exportAction = new QAction(tr("&Export Market"), this);
  exportAction->setToolTip(tr("Show the world export market."));
  // exportAction->setStatusTip(tr("Show the world export market."));
  exportAction->setEnabled(false);
  connect(exportAction,SIGNAL(triggered()),this,SLOT(ExMarket()));

  importAction = new QAction(tr("&Import Market"), this);
  importAction->setToolTip(tr("Show the world import market."));
  // importAction->setStatusTip(tr("Show the world import market."));
  importAction->setEnabled(false);
  connect(importAction,SIGNAL(triggered()),this,SLOT(ImMarket()));

  exitAction = new QAction(tr("&Quit"), this);
  exitAction->setIcon(QIcon("icons/window-close.png"));
  exitAction->setShortcut(tr("Ctrl+Q"));
  exitAction->setStatusTip(tr("Quit the program."));
  connect(exitAction,SIGNAL(triggered()),qApp,SLOT(quit()));

  tradeonoffAction = new QAction(tr("Open Economy"), this);
  tradeonoffAction->setIcon(QIcon("icons/national.png"));
  tradeonoffAction->setStatusTip(tr("Open the economy for trade."));
  tradeonoffAction->setCheckable(true);
  tradeonoffAction->setChecked(false);
  connect(tradeonoffAction,SIGNAL(triggered()),this,SLOT(EconOpenClosed()));

  hintlinesonoffAction = new QAction(tr("Show Hintlines"), this);
  hintlinesonoffAction->setIcon(QIcon("icons/helplinesOn.xpm"));
  hintlinesonoffAction->setStatusTip(tr("Show helplines (on/off)."));
  hintlinesonoffAction->setCheckable(true);
  hintlinesonoffAction->setChecked(true);
  connect(hintlinesonoffAction,SIGNAL(triggered()),this,SLOT(HintLinesOnOff()));

  tooltipsonoffAction = new QAction(tr("Show Tooltips"), this);
  tooltipsonoffAction->setStatusTip(tr("Show tooltips (on/off)."));
  tooltipsonoffAction->setCheckable(true);
  tooltipsonoffAction->setChecked(true);
  connect(tooltipsonoffAction,SIGNAL(triggered()),this,SLOT(TooltipsOnOff()));

  fixedpricemodelAction = new QAction(tr("&Fixed Price Model on/off"), this);
  fixedpricemodelAction->setIcon(QIcon("icons/FPM_gray.xpm"));             // see icon.cpp
  fixedpricemodelAction->setStatusTip(tr("Fixed price model (on/off)."));
  fixedpricemodelAction->setCheckable(true);
  fixedpricemodelAction->setChecked(false);
  connect(fixedpricemodelAction,SIGNAL(triggered()),this,SLOT(FixedPrice()));

  screenAction = new QAction(tr("&Window size (large/small)"), this);
  screenAction->setIcon(QIcon("icons/zoom-in.png"));
  screenAction->setStatusTip(tr("Show a large or small window."));
  connect(screenAction,SIGNAL(triggered()),this,SLOT(ReverseScreen()));

  adjustPAction = new QAction(tr("Adjust price level P"), this);
  adjustPAction->setIcon(QIcon("icons/adjustP.xpm"));                      // see icon.cpp
  // adjustP luft nicht mit StatusTip !?
  // adjustPAction->setStatusTip(tr("Adjust price level P"));

  adjustExAction = new QAction(tr("Adjust Ex"), this);
  adjustExAction->setIcon(QIcon("icons/adjustEx_gray.xpm"));
  // adjustExAction->setStatusTip(tr("Adjust autonomous export"));

  adjustMPAction = new QAction(tr("Adjust M and P"), this);
  adjustMPAction->setIcon(QIcon("icons/adjustMP_gray.xpm"));
  // adjustMPAction->setStatusTip(tr("Adjust money stock M and price level P (Z=dM)."));

  adjusteAction = new QAction(tr("Adjust e"), this);
  adjusteAction->setIcon(QIcon("icons/adjuste_gray.xpm"));
  // adjusteAction->setStatusTip(tr("Adjust exchange rate e."));

  growthAction = new QAction(tr("Simulate a growth process"), this);
  growthAction->setIcon(QIcon("icons/growth.xpm"));
  // growthAction->setStatusTip(tr("Simulate a growth process"));
  connect(growthAction,SIGNAL(triggered()),this,SLOT(GrowthSimulation()));

  printAction = new QAction(tr("Print"), this);
  printAction->setIcon(QIcon("icons/printer.png"));
  printAction->setStatusTip(tr("Print the scenario"));
  connect(printAction,SIGNAL(triggered()),this,SLOT(PrinterOutput()));

  IS_CAction = new QAction(tr("&IS-curve: consumption"), this);
  IS_CAction->setIcon(QIcon("icons/IS_C.xpm"));                            // see icon.cpp
  IS_CAction->setStatusTip(tr("IS-curve: Y=C+I+G."));
  IS_CAction->setCheckable(true);
  IS_CAction->setChecked(false);
  connect(IS_CAction,SIGNAL(triggered()),this,SLOT(ISCurveVersion()));

  IS_SAction = new QAction(tr("&IS-Curve: savings"), this);
  IS_SAction->setIcon(QIcon("icons/IS_S.xpm"));                            // see icon.cpp
  IS_SAction->setStatusTip(tr("IS-curve: S+T=I+G."));
  IS_SAction->setCheckable(true);
  IS_SAction->setChecked(true);
  connect(IS_SAction,SIGNAL(triggered()),this,SLOT(ISCurveVersion()));

  aboutmacroAction = new QAction(tr("&About the program"), this);
  aboutmacroAction->setIcon(QIcon("icons/help-about.png"));      
  aboutmacroAction->setStatusTip(tr("Anything you need to know about the program."));
  connect(aboutmacroAction,SIGNAL(triggered()),this,SLOT(about()));

  aboutQtAction = new QAction(tr("&About Qt"), this);
  aboutQtAction->setIcon(QIcon("icons/help-about.png"));      
  aboutQtAction->setStatusTip(tr("Anything you need to know about Qt."));
  connect(aboutQtAction,SIGNAL(triggered()),this,SLOT(aboutQt()));

  whatisAction = new QAction(tr("What is this"), this);
  whatisAction->setIcon(QIcon("icons/help-contents.png"));      
  whatisAction->setStatusTip(tr("Hints on buttons and fields."));
  connect(whatisAction,SIGNAL(triggered()),this,SLOT(whatsThis()));
}
void HBWidget::createMenus()
{
  fileMenu = menuBar()->addMenu(tr("&File"));
  fileMenu->addAction(resetAction);
  fileMenu->addAction(printAction);
  fileMenu->addAction(exportAction);
  fileMenu->addAction(importAction);
  fileMenu->addSeparator();
  fileMenu->addAction(exitAction);
  optionsMenu = menuBar()->addMenu(tr("&Options"));
  optionsMenu->addAction(tradeonoffAction);
  fileMenu->addSeparator();
  optionsMenu->addAction(fixedpricemodelAction);
  fileMenu->addSeparator();
  optionsMenu->addAction(screenAction);
  fileMenu->addSeparator();
  optionsMenu->addAction(hintlinesonoffAction);
  optionsMenu->addAction(tooltipsonoffAction);
  fileMenu->addSeparator();
  optionsMenu->addAction(IS_CAction);
  optionsMenu->addAction(IS_SAction);
  menuBar()->addSeparator();
  helpMenu = menuBar()->addMenu(tr("&Help"));
  helpMenu->addAction(aboutmacroAction);
  helpMenu->addAction(whatisAction);
  helpMenu->addAction(aboutQtAction);

  isOpen = false;       // starting with a closed economy
  FPM_ID = false;       // no Fixed Price Model at the beginning
  DrawHintLines = true; // we draw hint lines
  SmallScreen = true;   // on a small resolution
  ShowToolTips = false; 
  ShowExPostIdentity = true;
}
void HBWidget::createToolBars()
{
  HBToolBar = addToolBar(tr("File"));
  HBToolBar->addAction(exitAction);
  HBToolBar->addAction(resetAction);
  HBToolBar->addAction(screenAction);
  HBToolBar->addAction(hintlinesonoffAction);
  HBToolBar->addAction(tradeonoffAction);
  HBToolBar->addAction(fixedpricemodelAction);
  HBToolBar->addAction(adjustPAction);
  HBToolBar->addAction(adjustExAction);
  HBToolBar->addAction(adjustMPAction);
  HBToolBar->addAction(adjusteAction);
  HBToolBar->addAction(growthAction);
  HBToolBar->addAction(IS_CAction);
  HBToolBar->addAction(printAction);
  HBToolBar->addAction(whatisAction);
}
void HBWidget::createStatusBar()
{
  messageLabel = new QLabel();
  messageLabel->setMargin(3);
  messageLabel->setIndent(3); // Abstand zw. Text und linkem Rand
  messageLabel->setFrameStyle(QFrame::Box | QFrame::Raised);
  ShowProgress = new QProgressBar( statusBar() );
  ShowProgress->setRange( 0, 100 );
  ShowProgress->setValue( 0 );
  // QWhatsThis::add( ShowProgress, tr("QProgressBar"));

  // message : progess = 4 : 1
  statusBar()->addWidget(messageLabel, 4);
  //  statusBar()->setBackgroundColor( QColor( 180, 180, 180) );
  statusBar()->addWidget(ShowProgress, 1);
  updateStatusBar(tr("Ready"));
}
void HBWidget::updateStatusBar( const QString& text )
{
    messageLabel->setText( text );
}
void HBWidget::updateProgress( int progress )
{
  ShowProgress->setValue( progress );
}
void HBWidget::about()
{
  QMessageBox::information(
     this,
     tr("Macroeconomics"),
     tr("About MESSAGE"));
}
void HBWidget::aboutQt()
{
  QMessageBox::aboutQt( this, "About Qt - a C++ toolkit" );
}
void HBWidget::whatsThis()
{
  QWhatsThis::enterWhatsThisMode();
}
void HBWidget::LMMarketChanged()
{
  LMMarket->UpdateLMMarket(P_LCD->value()/1000.);
}
void HBWidget::ISMarketChanged()
{
  ISMarket->UpdateISMarket(e_LCD->value()/1000.,
			   P_LCD->value()/1000. );
}
void HBWidget::ISLMChanged( double YDEqNew )
{
  if ( isOpen == true ) {
    w2->WhatMarket(0, 
		   e_LCD->value()/1000.,
		   P_LCD->value()/1000.,
		   YDEqNew);
    w1->WhatMarket(1, 
		   e_LCD->value()/1000.,
		   P_LCD->value()/1000.,
		   YDEqNew);
  }
}
void HBWidget::aggrMarketChanged()
{
  aggrMarket->UpdateLaborMarket( K_LCD->value(), w_LCD->value()/1000. ); 
}
void HBWidget::Reset()
{
  keynes4->setChecked(true);
  ISMarket->InitialValues( 3 );  
  LMMarket->InitialValues( 3 );  
  ISLMMarket->InitialValues();  
  aggrMarket->InitialValues( 3 );
  ISMarket->InitialValues( 3 );  // correction of iEq, YDEq 
  LMMarket->InitialValues( 3 );  // correction of iEq, YDEq   
  messageLabel->setText( tr("Initial Values") );
}
void HBWidget::ExMarket()
{
  w2->show() ;
  w2->WhatMarket(0, 
		 e_LCD->value()/1000.,
		 P_LCD->value()/1000.,
		 1000.);
  messageLabel->setText( tr("Export market") );
}
void HBWidget::ImMarket()
{
  w1->show() ;
  w1->WhatMarket(1, 
		 e_LCD->value()/1000.,
		 P_LCD->value()/1000.,
		 1000.);
  messageLabel->setText( tr("Import market") );
}
void HBWidget::EconOpenClosed()
{
  // no growth simulation for open economies
  isOpen = Toggle(isOpen);
  if ( isOpen == false ) {
    messageLabel->setText( tr("Closed economy") );
    importAction->setEnabled(false);
    exportAction->setEnabled(false);
    tradeonoffAction->setIcon(QIcon("icons/national.png"));
    adjustMPAction->setIcon(QIcon("icons/adjustMP_gray.xpm"));
    adjustExAction->setIcon(QIcon("icons/adjustEx_gray.xpm"));
    adjusteAction->setIcon(QIcon("icons/adjuste_gray.xpm"));
    //Growth->setPixmap( QPixmap( GrowthPixmap ));
  }
  else {
    messageLabel->setText( tr("Open economy") );
    importAction->setEnabled(true);
    exportAction->setEnabled(true);
    tradeonoffAction->setIcon(QIcon("icons/international.png"));
    adjustMPAction->setIcon(QIcon("icons/adjustMP.xpm"));
    adjustExAction->setIcon(QIcon("icons/adjustEx.xpm"));
    adjusteAction->setIcon(QIcon("icons/adjuste.xpm"));
    //    Growth->setPixmap( QPixmap( GrowthPixmap_gray ));
  }
  ISMarket->OpenClosedEconomy( isOpen ) ;
}
void HBWidget::GrowthSimulation()
{
  if (isOpen==false) {
    Reset();
    repaint();
    ISMarket->GrowthValues();  
    aggrMarket->Growth(); 
  }
  else {
  QMessageBox::information(
     this,
     tr("Economic growth"),
     tr("Economic growth is implemented only for a closed economy."));
  }
}
void HBWidget::FixedPrice()
{
  FPM_ID = Toggle(FPM_ID);
  if (FPM_ID==false) {
    messageLabel->setText( tr("FPM off") );
    fixedpricemodelAction->setIcon(QIcon("icons/FPM_gray.xpm"));           // see icon.cpp
    aggrMarket->FPMChanged( FPM_ID ) ;
    ISLMMarket->FPMChanged( FPM_ID ) ;
    adjustPAction->setIcon(QIcon("icons/adjustP.xpm"));                    // see icon.cpp
    adjustPAction->setEnabled(true);
  }
  else {
    messageLabel->setText( tr("FPM on") );
    fixedpricemodelAction->setIcon(QIcon("icons/FPM.xpm"));                // see icon.cpp
    aggrMarket->FPMChanged( FPM_ID ) ;
    ISLMMarket->FPMChanged( FPM_ID ) ;
    adjustPAction->setIcon(QIcon("icons/adjustP_gray.xpm"));               // see icon.cpp
    adjustPAction->setEnabled(false);
  }
}
void HBWidget::HintLinesOnOff()
{
  DrawHintLines = Toggle(DrawHintLines);
  if ( DrawHintLines == false ) {
    messageLabel->setText( tr("Hint lines off") );
    hintlinesonoffAction->setIcon(QIcon("icons/helplinesOn.xpm"));
  }
  else {
    messageLabel->setText( tr("Hint lines on") );
    hintlinesonoffAction->setIcon(QIcon("icons/helplinesOff.xpm"));
  }
  LMMarket->HintLines(DrawHintLines) ;
  ISMarket->HintLines(DrawHintLines) ;
  ISLMMarket->HintLines(DrawHintLines) ;
  aggrMarket->HintLines(DrawHintLines) ;
}
void HBWidget::TooltipsOnOff()
{
  ShowToolTips = Toggle(ShowToolTips);
  if (ShowToolTips == true) {
    messageLabel->setText( tr("Tool tips on") );
    V__LCD->setToolTip(tr("TOOLTIP k"));
    M_LCD->setToolTip(tr("Nominal money supply"));
    c_LCD->setToolTip(tr("TOOLTIP c"));
    t_LCD->setToolTip(tr("TOOLTIP tax"));
    C0_LCD->setToolTip(tr("TOOLTIP Caut"));
    G_LCD->setToolTip(tr("TOOLTIP G")); 
    LMMarket->setToolTip(tr("TOOLTIP LM"));
    ISMarket->setToolTip(tr("TOOLTIP IS"));
    ISLMMarket->setToolTip(tr("TOOLTIP ISLM"));
    aggrMarket->setToolTip(tr("TOOLTIP Equi"));
    P_LCD->setToolTip(tr("TOOLTIP P"));
    e_LCD->setToolTip(tr("TOOLTIP e"));
    w_LCD->setToolTip(tr("TOOLTIP wmin"));
    K_LCD->setToolTip(tr("TOOLTIP K"));
    alpha_LCD->setToolTip(tr("TOOLTIP alpha"));
  }
  else {
    messageLabel->setText( "" );
    messageLabel->setText( "" );
    V__LCD->setToolTip( "" );
    M_LCD->setToolTip( "" );
    c_LCD->setToolTip( "" );
    t_LCD->setToolTip( "" );
    C0_LCD->setToolTip( "" );
    G_LCD->setToolTip( "" );
    LMMarket->setToolTip( "" );
    ISMarket->setToolTip( "" );
    ISLMMarket->setToolTip( "" );
    aggrMarket->setToolTip( "" );
    P_LCD->setToolTip( "" );
    e_LCD->setToolTip( "" );
    w_LCD->setToolTip( "" );
    K_LCD->setToolTip( "" );
    alpha_LCD->setToolTip( "" );
  }
}
void HBWidget::ReverseScreen()
{
  SmallScreen = Toggle( SmallScreen );
  if ( SmallScreen == false ) {
    SCALE = 1. ;
    hskip = int(SCALE * 10) ;
    WindowWidth = 1000;
    WindowHeight = 750;
#if __GNUG__ /* GNU C++ Compiler Linux */
    setFont( QFont( "helvetica", 12 ) ); 
#else        /* C++ for Mickysoft Windows */
    setFont( QFont( "arial", 10 ) ); 
#endif 
    setMinimumSize( WindowWidth, WindowHeight );
    setGeometry( 50, 50, WindowWidth, WindowHeight );
    messageLabel->setText( tr("Large Window") );
    screenAction->setIcon(QIcon("icons/zoom-in.png"));
  }
  else { 
    SCALE = 0.75 ;
    hskip = int(SCALE * 10) ;
    WindowWidth = 800;
    WindowHeight = 600;
#if __GNUG__ /* GNU C++ Compiler Linux */
    setFont( QFont( "helvetica", 10, 50 ) );
#else        /* C++ for Mickysoft Windows */
    setFont( QFont( "arial", 8, 50 ) ); 
#endif 
    setMinimumSize( WindowWidth, WindowHeight );
    setGeometry( 50, 50, WindowWidth, WindowHeight );
    messageLabel->setText( tr("Small Window") );
    screenAction->setIcon(QIcon("icons/zoom-out.png"));
  }
  WidgetPositions() ;
  LMMarket->Resolution(SmallScreen) ;
  ISMarket->Resolution(SmallScreen) ;
  ISLMMarket->Resolution(SmallScreen) ;
  aggrMarket->Resolution(SmallScreen) ;
  update();
}
void HBWidget::WidgetPositions()
{
  int tmp = int( SCALE * 320 );
  V__LCD->setGeometry(hskip,                    voffset, 75, 80 );
  M_LCD->setGeometry( hskip      +  (hskip+75), voffset, 75, 80 );

  c_LCD->setGeometry( WindowWidth-4*(hskip+75), voffset, 75, 80 );
  t_LCD->setGeometry( WindowWidth-3*(hskip+75), voffset, 75, 80 );
  C0_LCD->setGeometry(WindowWidth-2*(hskip+75), voffset, 75, 80 );
  G_LCD->setGeometry( WindowWidth-  (hskip+75), voffset, 75, 80 );
  LMMarket->setGeometry( hskip, V__LCD->y() + V__LCD->height() + 5, 
			 tmp, tmp);
  ISLMMarket->setGeometry( LMMarket->x() + LMMarket->width() + hskip, LMMarket->y(), 
			   tmp, int(SCALE * 170));
  ISMarket->setGeometry( ISLMMarket->x() + ISLMMarket->width() + hskip, 
			 LMMarket->y(), tmp, tmp);
  aggrMarket->setGeometry( LMMarket->x() + LMMarket->width() + hskip,
			   ISLMMarket->y()+ ISLMMarket->height()+ hskip,
			   tmp, tmp);
  P_LCD->setGeometry( aggrMarket->x()+ aggrMarket->width() + 10,
  		      WindowHeight - 2 * V__LCD->height() - 30, 75, 80);
  e_LCD->setGeometry( P_LCD->x()+P_LCD->width() + hskip, P_LCD->y(), 75, 80);
  K_LCD->setGeometry( P_LCD->x(), P_LCD->y() + P_LCD->height(), 75, 80);
  alpha_LCD->setGeometry( K_LCD->x()+K_LCD->width() + hskip, 
			   K_LCD->y(), 75, 80);
  w_LCD->setGeometry( aggrMarket->x() -75 -hskip, K_LCD->y(), 75, 80);
  rb->setGeometry(hskip, WindowHeight - 140 - 35, 160, 140);
}
void HBWidget::PrinterOutput()
{
    QPrinter *printer = new QPrinter;
    QPrintDialog *printDialog = new QPrintDialog(printer, this);
    printer->setPaperSize( QPrinter::A4 );
    // printer->setFullPage( true );
    printer->setOrientation( QPrinter::Landscape );
    // printer->setPrintRange( QPrinter::AllPages );
    if (printDialog->exec() == QDialog::Accepted) {
        QPainter p(printer);
        QPixmap pm = QPixmap::grabWidget(this);
        p.drawPixmap(0, 0, pm);
    };
}
void HBWidget::KeynesMessage() 
{
  int KeynesCase = 3;
  if ( keynes1->isChecked() == true ) KeynesCase = 0;
  if ( keynes2->isChecked() == true ) KeynesCase = 1;
  if ( keynes3->isChecked() == true ) KeynesCase = 2;
  aggrMarket->InitialValues(KeynesCase);
  ISMarket->InitialValues(KeynesCase);
  LMMarket->InitialValues(KeynesCase);
  ISLMMarket->InitialValues();
  ISMarket->InitialValues(KeynesCase);
  LMMarket->InitialValues(KeynesCase);
  switch (KeynesCase) {
  case 0: messageLabel->setText( tr("Liquidity Trap") );
          break ;
  case 1: messageLabel->setText( tr("Investment Trap") );
          break ; 
  case 2: messageLabel->setText( tr("Wage Rate Rigidities") );
          break ;
  case 3: messageLabel->setText( tr("Usual Case") );
          break ;
  }
}
void HBWidget::wminClicked()
{
  keynes3->setChecked(true);
}
void HBWidget::ISCurveVersion()
{
  ShowExPostIdentity = Toggle(ShowExPostIdentity);
  if ( ShowExPostIdentity == true ) {
    messageLabel->setText( tr("S+T=I+G") );
    IS_CAction->setIcon(QIcon("icons/IS_C.xpm"));                          // see icon.cpp
    IS_CAction->setChecked(false);
    IS_SAction->setChecked(true);
  }
  else {
    messageLabel->setText( tr("Y=C+I+G") );
    IS_CAction->setIcon(QIcon("icons/IS_S.xpm"));         // see icon.cpp
    IS_CAction->setChecked(true);
    IS_SAction->setChecked(false);
  }
  ISMarket->SavingsConsumption(ShowExPostIdentity) ;
}
