#ifndef EQUILIBRIUM_H
#define EQUILIBRIUM_H

#include <QtWidgets/QWidget>
#include <QtWidgets/QMessageBox>
#include <QtGui/QMouseEvent>
#include <QThread>

class Sleeper : public QThread
{
public:
    static void msleep(unsigned long msecs){QThread::msleep(msecs);}
};

class QPainter ;
class CommodityMarket;

class Equilibrium : public QWidget
{
  Q_OBJECT
  public:
    Equilibrium( QWidget *parent=0 );
    void DrawAxes( QPainter * );
    void paintLaborMarket( QPainter * );
    void paintDemandSide( QPainter * );
    void UpdateLaborMarket( double, double ); 
    void Growth();
    void DrawCurve( QPainter * , QPolygon, QColor );  // //alt QPointarry
  public slots:
    void set_K( int );
    void set_P( int );
    void set_w( int );
    void set_alpha( int );
    void HintLines( bool );
    void Resolution( bool );
    void InitialValues( int );
    void FPMChanged( int );
    void Adjustment( int );
    void AdjustPriceLevel();
    void UpdateEquilibrium( double *, bool ); 
  protected:
    void paintEvent( QPaintEvent * );
    void mousePressEvent( QMouseEvent * );
    void mouseMoveEvent( QMouseEvent * );
  signals:
    void KChanged( int );
    void PChanged( int );
    void wChanged( int );
    void alphaChanged( int );
    void GChanged( int );
    void CautChanged( int );
    void IautChanged( double, double, double );
    void MChanged( int );
    void wminClicked();
    void explain( const QString& );
    void updateProgressBar( int );
  private:
    int Capital_h, K_h, P_h, w_h, alpha_h, KeynesCase, progress; 
    // Money market
    double k, M, P, iMIN2, iMAX2, LS0, delta, YatiMINonLM, YatiMAXonLM;
    // Commodity market
    double rho, s, IatiMIN, iMAX1, autDemand, YatiMAXonIS;
    // Labor Market
    double alpha, beta, Kalpha, wMin, NDatPD, NSatPD, ERate, gamma ;
    double Yscaling, Nscaling, Pscaling, wscaling, SCALE;
    int CENTER ;
    double YSKeynes, PKeynes, NKeynes, YSEqatPD;
    double Ycritical1, Ycritical2, YDEq, YSEq, PEq, Phint;
    QPoint Origin;
    bool HINTFLAG, OpenEconomy, FPM_ID;
    QMessageBox *helpmessage;
    CommodityMarket *ISMarket;
};
#endif 
