/*
 * graphical representation of the 
 * IS-LM field (internal and external equilibrium)
 * initial situation and after changes of parameters (updates)
 */
#define RangeOf(x,a,b)  if ( x < a ) x = a; if ( x > b )  x = b
template <class X> X Min(X x, X y) { return (x<=y) ? x : y ;}
#include "IS-LM.h"

#include <QPainter>
#include <QPolygon>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "ISMarket.h"

ISLMField::ISLMField( QWidget *parent )
        : QWidget( parent ) 
{
  FPM_ID = false;
  Resolution( true );   // must be first
  HINTFLAG = true ;     // paint gray lines
  OpenEconomy = false;  // default is a closed economy
  s = 0.58;             // s = 1.- c * (1.-tax);
  iMAX1=0.22; iMIN1=0.01;
  rho=7619.;  autDemand = 1856; YatiMAXonIS=303.;
  iMAX2=0.20; iMIN2=0.02;                
  k = 0.07;   delta= 500.;  LS0  = 100.;  M = 109.;
  e_h = 1;
  ERate = 1.; 
  P = 1.; M=110.;
  iEq  = 0.151;
  YDEq = 1219.8;
  YMAX = 2000. * 160./150.;
  points = new QPoint[300] ;
}
void ISLMField::paintEvent( QPaintEvent * ) 
{
  QPainter r( this ); 
  DrawAxes( &r );
  paintISLM( &r ) ; 
  /*
   *  Finally, a signal to Equilibrium.cpp
   */
  double VarList[15];
  VarList[0] = k;
  VarList[1] = LS0;
  VarList[2] = delta;
  VarList[3] = M;
  VarList[4] = P;
  VarList[5] = autDemand;
  VarList[6] = s;
  VarList[7] = rho;
  VarList[8] = iMAX1;
  VarList[9] = iMAX2;
  VarList[10] = ERate;
  VarList[11]= YDEq;
  VarList[12]= YatiMINonLM; 
  VarList[13]= YatiMAXonLM;
  VarList[14]= YatiMAXonIS;
  emit NewEquilibrium( VarList, OpenEconomy); 
}
void ISLMField::DrawAxes( QPainter *r )
{
  r->setBrush(QBrush("#fffff0"));  
  r->setPen(Qt::black);
  r->drawRect( 0, 0, 1.999 * CENTER, int(SCALE * 169) );           
  r->setBrush(Qt::NoBrush); 
  r->drawText( int(SCALE * 165), int(SCALE * 15), "i" );
  r->drawText( int(SCALE * 300), int(SCALE * 155), "Y" );
  r->drawLine( CENTER, CENTER, int(SCALE * 310), CENTER);
  r->drawLine( CENTER, CENTER, CENTER, int(SCALE * 10));  
#if __GNUG__ /* GNU C++ Compiler Linux */
  r->setFont( QFont( "helvetica", (SCALE==0.75) ? 10 : 12, 25 ) ); 
#else        /* C++ for Mickysoft Windows */
  r->setFont( QFont( "arial", (SCALE==0.75) ? 8 : 11, 25 ) ); 
#endif 
}
void ISLMField::paintISLM( QPainter *r ) 
{
  QString HBString; 
  QPoint p1,p2;
  int tmp;
  /*
   *   Equilibrium
   */
  if (P==0) P=.000000001;
  if (k==0) k=.000000001; 
  YatiMAXonLM = ((M/P)-LS0+delta*iMAX2)/k;
  YatiMINonLM = ((M/P)-LS0+delta*iMIN2)/k;

  HBString.sprintf( " PD  =%7.3f ", P);
  r->drawText( int(SCALE * 25), int(SCALE * 100), HBString );
  r->setPen(Qt::black);
  if (YatiMAXonIS>YatiMAXonLM) {
    r->setPen(Qt::red);
    r->drawPixmap( 2*CENTER - 38, int(SCALE * 5), QPixmap("icons/bug.png"));
    QMessageBox::critical(this,tr("ERROR IS-LM"),
			 tr("MESSAGE ERROR IS-LM"),
			 QMessageBox::Cancel);
    r->setPen(Qt::black);
  }
  else {
    YDEq = YDEqCalc() ;  
    r->setPen(Qt::black);
    HBString.sprintf( " i*  =%7.2f%% ", iEq * 100.);
    r->drawText( int(SCALE * 25), int(SCALE * 60), HBString );
    HBString.sprintf( " YD* =%7.1f ", YDEq);
    r->drawText( int(SCALE * 25), int(SCALE * 80), HBString );
    if (HINTFLAG==true) {
      emit EqValuesChanged( iEq, YDEq );
      r->setPen(Qt::lightGray);
      p1 = QPoint( 0,-int( iscaling * iEq) ) + Origin ;
      if (YDEq<YMAX) {
	p2 = QPoint( int(Yscaling * YDEq),-int(iscaling * iEq) ) + Origin ;
	r->drawLine( p1, p2 );
	p1 = QPoint( int(Yscaling * YDEq), 0 ) + Origin ;
	r->drawLine( p2, p1 );
      }
      else {
	p2 = QPoint( CENTER,-int(iscaling * iEq) ) + Origin ;
	r->drawLine( p1, p2 );
      }
    }
  }
  /*
   *   IS curve + Z0 curve (both of them are known from ISMarket.cpp)
   */
  if ( OpenEconomy == true ) {
    QPolygon ISCurve(150) ; 
    QPolygon Z0Curve(150) ; 
    for (int i=0; i<150; i++ ) {
      Z0Curve.setPoint(i,int(Z0Coor[0][i]),int(Z0Coor[1][i]));
      ISCurve.setPoint(i,int(ISCoor[0][i]),int(ISCoor[1][i]));
    }
    DrawCurve( r, ISCurve, Qt::red ) ;
    DrawCurve( r, Z0Curve, QColor(85, 26, 139) ) ;     
    
    r->setPen(QColor(85, 26, 139));
    r->drawText( int(SCALE * 25), int(SCALE * 15), tr("Z=0 curve") );
    HBString.sprintf( " Z = %5.1f ", 
	      ISMarket->BoPA(YDEq,4500.,P,2.,ERate) 
	      + 3000. * (iEq - 0.09) );
    r->drawText( int(SCALE * 25), int(SCALE * 35), HBString );
    r->setPen(Qt::black);
  }
  else { // closed economy
    double YatiMIN = (autDemand-rho*iMIN1)/s;
    r->setPen(Qt::red);
    if (iscaling * iMAX1< CENTER) {
      p1 = QPoint(int(Yscaling * YatiMAXonIS), -CENTER )  + Origin ;
      p2 = QPoint(int(Yscaling * YatiMAXonIS), int(-iscaling * iMAX1)) + Origin ;
      r->drawLine( p1, p2 ); 
    }
    else {
      p2 = QPoint(int(Yscaling * ( autDemand - rho * .3 * 160. /150. ) / s), 
		  -CENTER) 
	 + Origin ;
    }
    tmp = Min( int(Yscaling * YatiMIN), CENTER );
    if (Yscaling * YatiMIN<CENTER) {
      p1 = QPoint( tmp,-int(iscaling * iMIN1) )  + Origin ;
      r->drawLine( p2, p1 );
      p2 = QPoint( CENTER,-int(iscaling * iMIN1) )  + Origin ;
      r->drawLine( p1, p2 );
    }
    else {
      p1 = QPoint( tmp, int(iscaling * (YMAX * s - autDemand)/rho) )  + Origin ;
      r->drawLine( p2, p1 );
    }
    r->setPen(Qt::black);
  }
  double LS_kink = LS0 - delta * iMIN2;
  r->setPen(QColor(34, 139,  34));
  if (M/P<LS_kink) { 
    p1 = QPoint( 0, int(iscaling * (M/P-LS0)/delta) )  + Origin ;
  }
  else { 
    p2 = QPoint( 0, -int(iscaling * iMIN2) )  + Origin ;
    tmp = Min( int(Yscaling * (M/P-LS_kink)/k), CENTER );
    p1 = QPoint( tmp,-int(iscaling * iMIN2) )  + Origin ;
    r->drawLine( p2, p1 );
  }
  tmp = Min( int(Yscaling * (M/P)/k), CENTER ) ;
  if (Yscaling * (M/P)/k <= CENTER ) {
    p2 = QPoint( tmp,-int(iscaling * iMAX2) )  + Origin ;
    r->drawLine( p1, p2 ); 
    p1 = QPoint( tmp,-CENTER )  + Origin ;
    r->drawLine( p2, p1 ); 
  }
  else {
    p2 = QPoint( tmp,-int(iscaling * (M/P - LS0 - k * YMAX)/(-delta)) ) + Origin ;
    r->drawLine( p1, p2 );
  }
}
void ISLMField::ISChanged( double sNew, 
			   double autDemandNew, 
			   double rhoNew, 
			   double iMAXNew,
			   bool OpenEconomyNew,
			   double YatiMAXNew,
			   double eNew) 
{
  s=sNew;
  autDemand=autDemandNew;
  rho=rhoNew;
  iMAX1=iMAXNew;
  OpenEconomy=OpenEconomyNew;
  YatiMAXonIS=YatiMAXNew;
  ERate=eNew;
  update();
}
void ISLMField::Z0CurveChanged( double *ZList, 
				double *ISList ) 
{
  for (int i=0; i<150; i++ ) {
    Z0Coor[0][i]=*(ZList+i);
    Z0Coor[1][i]=*(ZList+i+150);
    ISCoor[0][i]=*(ISList+i);
    ISCoor[1][i]=*(ISList+i+150);
  }
  update();
}
void ISLMField::LMChanged( double kNew, 
			   double MNew, 
			   double PNew,
			   double deltaNew,
			   double LS0New,
			   double iMAXNew) 
{
  k=kNew;
  M=MNew;
  P=PNew;
  delta=deltaNew;
  LS0=LS0New;
  iMAX2=iMAXNew;
  update();
}
void ISLMField::HintLines( bool OnOffNew ) 
{
  HINTFLAG=OnOffNew;
  update();
}
void ISLMField::InitialValues() 
{
  e_h =  600; 
  set_e(e_h);
}
void ISLMField::Resolution( bool LowResolution ) 
{
  SCALE = (LowResolution == true) ? 0.75 : 1. ;
  CENTER = int(SCALE * 160) ;
  Origin = QPoint( CENTER, CENTER );
  Yscaling = SCALE * 150. / 2000. ;
  iscaling = SCALE * 150. / .3 ;
  update();
}
void ISLMField::AdjustmentMP( ) 
{
  if (OpenEconomy == true) {
    if (FPM_ID == true) {
      QMessageBox::warning(this,tr("FPM"),
			 tr("MESSAGE FPM"),
			 QMessageBox::Ok);
      return ; 
    }
    QString HBString;
    emit AdjustPriceLevel( 0 );  
    for (int i=0;i<10;i++) { 
      double Z = ISMarket->BoPA(YDEq,4500.,P,2.,ERate) //Trade Balance
	+ 3000. * (iEq - 0.09) ;                       //Net Capital Imports 
      HBString.sprintf("Running ... %d, Z=%f, M=%f",i,Z,M); 
      emit explain( HBString );
      if ( fabs(Z) > 1.5 ) {
	M = M + Z / 5. ;  // This is not dM=Z !
	if (M<0) {
	  HBString.sprintf("M = %f negativ! Abbruch!",M);
	  emit explain( HBString );
	  return;
	}
	emit MChanged( int(M * 10.) ); // -> LMMarket -> M_LCD
	repaint();                     // Die Simulation luft sonst nicht!
	emit AdjustPriceLevel( 1 ); 
      }
    }
    HBString.sprintf("Finished ...");
    emit explain( HBString );
    emit updateProgressBar( 0 ); 
  }
}
void ISLMField::AdjustmentERate() 
{
  double Z=0., Z_old=0.;
  int i=0;
  QString HBString;
  if (OpenEconomy == true) {
    HBString.sprintf("Running ...");
    emit explain( HBString );
    emit AdjustPriceLevel( 0 );
    Z = ISMarket->BoPA(YDEq,4500,P,2,ERate) + 3000. * (iEq-0.09) ;
    if ( Z < -0.5 ) { Z_old = Z-2.; }
    if ( Z > 0.5 )  { Z_old = Z+2.; }
    while ((fabs(Z) > 0.5) && (i<20)) {
      Z = ISMarket->BoPA(YDEq,4500,P,2,ERate) + 3000. * (iEq-0.09) ;
      HBString.sprintf("Running ... Z=%f",Z);
      emit explain( HBString );
      if (fabs(Z)>1.5) {
	  ERate = ERate - (Z/2000.) ; 
	  set_e( int(ERate * 1000.) );
	  emit AdjustPriceLevel( 1 );
/*
ein Test auf eine anomale Reaktion der ZB wird nicht durchgefhrt
*/
      }
      /*      
      if ( Z < -1.5 ) {               // Abwertung wegen Z<0
	if ( Z > Z_old) {             // dZ/de>0
	  ERate = ERate - (Z/2000.) ; 
	  set_e( int(ERate * 1000.) );
	  emit AdjustPriceLevel( 1 );
	}
	else { 
	  i=25; 
	  HBString.sprintf("dZ/de<0 (a)");
	  emit explain( HBString );
	  emit updateProgressBar( 0 ); 
	}
      }
      if ( Z > 1.5 ) {                // Aufwertung wegen Z>0
	if ( Z < Z_old) {             // dZ/de>0
	  ERate = ERate - (Z/2000.) ; // 
	  set_e( int(ERate * 1000.) );
	  emit AdjustPriceLevel( 1 );
	}
	else { 
	  i=25; 
	  HBString.sprintf("dZ/de<0 (b)");
	  emit explain( HBString );
	  emit updateProgressBar( 0 ); 
	}
      }
      */
      Z_old = Z;
      i = i+1;
    }
    if ( (i==20) || (fabs(Z) <= 1.5) )  {
      HBString.sprintf("Finished ..."); 
      emit explain( HBString );
      emit updateProgressBar( 0 ); 
    }
  }
}
void ISLMField::DrawCurve ( QPainter *r, 
			    QPolygon Z, 
			    QColor color) 
{ 
  r->setPen(color);
  Z.translate ( CENTER, CENTER );
  r->drawPolyline( Z ); 
  r->setPen(Qt::black);
}
void ISLMField::mousePressEvent( QMouseEvent *e ) 
{
  QString HBString, tmpX, tmpY;
  double Yclick, iclick, DY, Di, Ydyn, idyn;
  int i, count = 0;
  update();                                // erase old points
  QPainter paint( this );
  if ( ( e->button() == Qt::LeftButton ) || ( e->button() == Qt::RightButton ) ) {
    HBString.sprintf( " Mausposition = (%d,%d) ", e->x(),e->y() );
    Ydyn = Yclick = ( e->x() - CENTER ) / Yscaling ;
    idyn = iclick = ( CENTER - e->y() ) / iscaling ;
    if ( e->x() > CENTER ) {
      tmpX.sprintf( ", Y = %7.1f",  Yclick );  HBString += tmpX ;
    }
    if ( e->y() < CENTER ) {
      tmpY.sprintf( ", i = %7.2f%%", iclick ); HBString += tmpY ;
    }
    if ( (e->x() > CENTER) && (e->y() < CENTER) ) {
      paint.drawPoint( e->pos() );
      if ( OpenEconomy == false ) {
	points[0] = e->pos();
	for ( count=1; count<300; count++ ) {
	  if ( idyn>iMIN1 ) {
	    if ( Ydyn <= (autDemand-rho*iMAX1)/s)
	      DY = -5.;
	    else 
	      DY = 0.03 * ( Ydyn - (autDemand-rho*idyn)/s );
	  }
	  else DY = -10. ;
	  if ( Ydyn<=YatiMAXonLM ) 
    	       Di = 0.01 * ( -(M/P-k*Ydyn-LS0)/delta - idyn ) ;
	  else Di = 0.05 ;
	  Ydyn = Ydyn - DY;
	  idyn = idyn + Di;
	  points[count] = QPoint( int(Yscaling * Ydyn), 
				 -int(iscaling * idyn) ) + Origin ;
	}
	for ( i=0; i<count-1; i++ ) {		
	  paint.drawLine( points[i], points[i+1] ); // draw line
	}
      }
    }
    emit explain( HBString );
  }
}
double ISLMField::YDEqCalc() 
{
  double YDSol = 0;
  if ( OpenEconomy==true ) { 
    double y0,y1,y2,f0,f1,f2,CAB0,CAB1,CAB2, 
           fix = -autDemand+(rho/delta)*(LS0-M/P) ;
    int REPEAT = true;
    y0 = 0. ;
    CAB0 = ISMarket->BoPA(y0,4500.,P,2.,ERate);
    f0 = ( (s+rho*k/delta) * y0 ) - CAB0 + fix ;
      y1 = (2150. <= YatiMAXonLM) ? YatiMAXonLM : 2150. ;
      CAB1 = ISMarket->BoPA(y1,4500.,P,2.,ERate);
      f1 = ( (s+rho*k/delta) * y1 ) - CAB1 + fix ;
      if (f0 * f1 < 0.)
	{ while ( REPEAT == true ) 
	  { y2 = (y0 * f1 - y1 * f0) / (f1 - f0);
            CAB2 = ISMarket->BoPA(y2,4500.,P,2.,ERate);
	    f2 = ( (s+rho*k/delta) * y2 ) - CAB2 + fix ;
	    if ( fabs(f2) < 0.2 ) {
	      YDSol = y2 ; 
	      emit YDEqChanged( YDSol ) ; // import & export market
	      REPEAT = false ;
	    } 
	    else {
	      if (f2 * f0 < 0. ) 
		{ y1 = y2; f1 = f2; }
	      else 
		{ y0 = y2; f0 = f2; }
	    }
	  }
	}
      else { YDSol = -1000.; }
  }
  else { // closed economy
    YDSol = ( (M/P) - LS0 + (delta/rho) * autDemand )
          / ( k + (delta/rho) * s );
  }
  iEq = - ( (M/P) - LS0 - k * YDSol ) / delta ;
  /*
   *   classical region
   *   liquidity trap
   *   investment trap
   */
  if ( YDSol>YatiMAXonLM ) { 
    YDSol = YatiMAXonLM; 
    iEq = -( s * YDSol - autDemand ) / rho;     
    if ( OpenEconomy == true ) 
      iEq += ISMarket->BoPA(YDSol,4500.,P,2.,ERate)/rho; 
  }
  if ((YDSol<YatiMINonLM ) && (YDSol>0)) {  
    iEq = iMIN2; 
    if ( OpenEconomy == true ) 
      YDSol = ISMarket->Y(iMIN2,4500.,P,2.,ERate,s,autDemand,rho);
    else 
      YDSol = (autDemand-rho*iMIN2)/s ;
  }
  if ( YDSol<=YatiMAXonIS ) { 
    YDSol = YatiMAXonIS; 
    iEq =-( M/P - LS0 - k * YDSol ) / delta; 
  }
  return YDSol ;
}
void ISLMField::set_e( int tmp ) 
{
  RangeOf(tmp,60,10000);
  if ( fabs(ERate-tmp/1000.)<0.00001 ) return; // don't play ping-pong with LCD
  ERate = tmp / 1000. ;
  emit eChanged( tmp ); // message to LCD
  update();
}
void ISLMField::FPMChanged( int ID )
{
  FPM_ID = ID;
}
