/*
 * graphical representation of the 
 * COMMODITY MARKET
 * initial situation and after changes of parameters (updates)
 */
#define RangeOf(x,a,b)  if ( x < a ) x = a; if ( x > b )  x = b

#include "ISMarket.h"
#include <QPainter>
#include <QPolygon>
#include <QRect>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

 double ae1=0.0080854, ce1=0.07887, 
        ae2=0.0153333, ce2=-5.14365;
 const double 
    be1=0.00002401, de1=0.0022371,
    be2=0.00001,    de2=0.0044291,
    ai1=0.0029902, bi1=0.000006536, ci1= 0.695, di1=0.0006,
    ai2=0.0028239, bi2=0.00000179,  ci2=-0.245, di2=0.00058;
 double Z=0., DELTA1=0., DELTA2=0.;

CommodityMarket::CommodityMarket( QWidget *parent )
        : QWidget( parent ) 
{
  Resolution( true );  // must be first
  HINTFLAG = true;
  InViewOfSavings = true;
  OpenEconomy = false ;
  c    = .7;  // marginal propensity to consume
  tax  = .4;  // tax rate
  Caut =140.; // autonomous consumption
  G    = 50.; // government expenditure
  YRand = 2150. ;
  YMAX  = 2000. * 160./150. ;
  P     = 1.;
  ERate = 2.;
  ihint = 0.151;
  Yhint = 1219.8;
  InitialValues( 3 );
}
void CommodityMarket::paintEvent( QPaintEvent * ) 
{
  s = 1. - c * (1.-tax);      // S+T=I+G

  QPainter r( this );    
  DrawAxes( &r );
  paintCommodityMarket( &r ) ; 
  emit ISMarket2ISLM(s,autDemand,rho,iMAX,OpenEconomy,YatiMAX,ERate);
}
void CommodityMarket::DrawAxes( QPainter *r )
{
  r->setBrush(QBrush("#fffff0"));  
  r->setPen(Qt::black);
  r->drawRect( 0, 0, 1.999 * CENTER, 1.999 * CENTER );
  r->setBrush(Qt::NoBrush); 
  /*
    HBpixmap represents inflationary and deflationary gaps.
   */
  r->drawPixmap(QPoint(0,0),HBpixmap);
  r->drawRect( 0, 0, 1.999 * CENTER, 1.999 * CENTER );
  r->drawLine( int(SCALE * 10), CENTER, int(SCALE * 310), CENTER );
  r->drawLine( CENTER, int(SCALE * 10), CENTER, int(SCALE * 310) );
#if __GNUG__ /* GNU C++ Compiler Linux */
  r->setFont( QFont( "helvetica", (SCALE==0.75) ? 10 : 12, 25 ) ); 
#else        /* C++ for Mickysoft Windows */
  r->setFont( QFont( "arial", (SCALE==0.75) ? 8 : 11, 25 ) ); 
#endif 
  r->drawText( int(SCALE * 165), int(SCALE * 15),  "i" );
  r->drawText( int(SCALE * 300), int(SCALE * 155), "Y" );
  if ( InViewOfSavings==true ) {
    if ( OpenEconomy == true ) 
      r->drawText( CENTER+int(SCALE * 5), int(SCALE * 310), "S+T+Im-Ex" );
    else 
      r->drawText( CENTER+int(SCALE * 5), int(SCALE * 310), "S+T" );
  }
  else {
    if ( OpenEconomy == true ) {
      r->drawText( int(SCALE * 112), int(SCALE * 310), "I+G" );
      r->drawText( CENTER+int(SCALE * 5), int(SCALE * 310), "C+I+G+Ex-Im" );
    }
    else {
      r->drawText( int(SCALE * 130) - 3, int(SCALE * 310), "I+G" );
      r->drawText( CENTER+int(SCALE * 5), int(SCALE * 310), "C+I+G" );
    }
  } 
  if ( OpenEconomy == true ) {
    r->drawText( 2,  int(SCALE * 155), "I+G" );
    r->setPen(Qt::red);
    r->drawText( int(SCALE * 200), int(SCALE * 15), tr("IEx-SIm curve") );
  }
  else {
    r->drawText( 2, int(SCALE * 155), "I+G" );
    r->setPen(Qt::red);
    r->drawText( int(SCALE * 200), int(SCALE * 15), tr("IS curve") );
  }
  r->setPen(Qt::black);
}
void CommodityMarket::paintCommodityMarket( QPainter *r ) 
{
  double YD1 ; 
  QPoint p1,p2,p3;
  autDemand = Caut + I0 + Iaut + G ;
  double IGhint = I(ihint) + G ; // ihint is calculated in IS-LM.cpp
  int temp1 = (SCALE < 1.) ? 120 : 160 ;
  if ( InViewOfSavings == true ) {
    /*
     *   savings + tax + imports - exports
     */
    if ( OpenEconomy == true ) {
      /*
       *  number of points to be shown 
       */
      QPolygon c1( temp1 ); // alt QPointArray
      double STCurve = 0. ;
      for (int i=0; i<temp1; i++ ) {
	YD1 = i / Yscaling ;
	STCurve = S(YD1) - BoPA(YD1,4500,P,2,ERate) ; 
	if (STCurve < 0.) { STCurve = 0.; } 
	c1.setPoint(i, i, int(Yscaling * STCurve));
      }
      ZEICHNEN( r, c1, Qt::black ) ; 
    }
    else { // closed economy
      p1 = Yscaling * QPoint( int(Caut/s), 0             ) + Origin;
      p2 = Yscaling * QPoint( int(YRand),  int(S(YRand)) ) + Origin;
      r->drawLine(p1,p2);
    }
  }
  else { 
    /*
     *   Y = C + I + G + Ex - Im = C(Y) + I(i) + G + Ex - Im
     */
    p1 = Origin; 
    r->drawLine(Origin, p1 + Origin);
    if ( OpenEconomy == true ) {
      QPolygon c2( temp1 );
      double CICurve = 0. ;
      for (int i=0; i<temp1; i++ ) {
	YD1 = i / Yscaling ;
	CICurve = C(YD1) + IGhint + BoPA(YD1,4500,P,2,ERate) ; 
	if (CICurve < 0.) { CICurve = 0.; } 
	c2.setPoint(i, i, int(Yscaling * CICurve));
      }
      ZEICHNEN( r, c2, Qt::black ) ; 
    }
    else { // closed economy      
      p1 = Yscaling * QPoint(0,          int(IGhint + Caut)    ) + Origin;
      p2 = Yscaling * QPoint(int(YRand), int(IGhint + C(YRand))) + Origin ;
      r->drawLine(p1,p2);
    }
  }
  /*
   *  equilibrium I+G=S+T or I+G+Ex=S+T+Im
   */
  r->drawLine( CENTER, CENTER, 0, 2 * CENTER); 
  /*
   *  investment + government expenditure
   */
  p1 = QPoint(int(-Yscaling * YMAX),              int( -iscaling * iMIN)) 
     + Origin;
  p2 = QPoint(int(-Yscaling * (IatiMIN+G + Iaut)),int( -iscaling * iMIN)) 
     + Origin;
  r->drawLine(p1,p2); 
  if (iscaling * iMAX <= CENTER ) {
    p1 = QPoint( int(-Yscaling * (G + Iaut)), int(-iscaling * iMAX) ) 
       + Origin ;
    r->drawLine( p2, p1 );
    p2 = QPoint( int(-Yscaling * (G + Iaut)), -CENTER ) 
       + Origin ;
    r->drawLine( p1, p2 ); }
  else {
    p1 = QPoint( int(-Yscaling * (G + Iaut+I0-rho*.3*160./150)), -CENTER ) 
       + Origin ;
    r->drawLine( p2, p1 );
  }
  /*
   *  IS curve ; open economy (Z=0 curve)
   */
  if ( OpenEconomy == true ) {
    QPolygon a1( 150 );
    QPolygon a2( 150 );
    double ISCurve = 0. , Z0Curve = 0.;
    /*
     *   calculate smallest Y
     */
    YatiMAX = Y(iMAX,4500.,P,2.,ERate,s,autDemand,rho);
    double ZList[2][150], ISList[2][150];
    for (int i=0; i<150; i++ ) {
      YD1 = i / Yscaling ;
      if ( YD1<YatiMAX ) {
	a1.setPoint(i, i, int(-SCALE*165.)); 
	Z0Curve = - BoPA(YatiMAX,4500,P,2,ERate)/3000. + 0.09 ;
	a2.setPoint(i, int(Yscaling * YatiMAX), int(-iscaling * Z0Curve)); 
	ISList[0][i] = i;
	ISList[1][i] = -SCALE*165.;
	ZList[0][i] = Yscaling * YatiMAX;
	ZList[1][i] = -iscaling * Z0Curve ;
      }
      else {
	ISCurve = -(s * YD1-autDemand-BoPA(YD1,4500,P,2,ERate))/rho; 
	Z0Curve = - BoPA(YD1,4500,P,2,ERate)/3000. + 0.09 ;
	// 0.09 = i2 = foreign interest rate
	// 3000 -->    NCI = 3000 (i - i2 ) 
	//	if (ISCurve>.22) { ISCurve = .4; }
	if (ISCurve<.01) { ISCurve = 0.01; }
	a1.setPoint(i, i, int(-iscaling * ISCurve)); 
	a2.setPoint(i, i, int(-iscaling * Z0Curve)); 
	ISList[0][i] = ZList[0][i] = i;
	ISList[1][i] = -iscaling * ISCurve ;
	ZList[1][i]  = -iscaling * Z0Curve ;
      }
    }
    ZEICHNEN( r, a1, Qt::red ) ; 
    ZEICHNEN( r, a2, QColor(85, 26, 139) ) ; 
    emit ZCurve2ISLM( *ZList, *ISList ) ;
  }
  else { 
  /*
   *  IS curve ; closed economy
   */
    double YatiMIN = ( autDemand - rho * iMIN ) / s;
    YatiMAX = ( autDemand - rho * iMAX ) / s;
    r->setPen(Qt::red);
    if (iscaling * iMAX<CENTER) {
      p1 = QPoint(int(Yscaling * YatiMAX),-CENTER)               + Origin ;
      p2 = QPoint(int(Yscaling * YatiMAX),-int(iscaling * iMAX)) + Origin ;
      r->drawLine(p1,p2); 
    }
    else {
      p2 = QPoint(int(Yscaling * ( autDemand - rho * .3 * 160. /150. ) / s), 
		  -CENTER) + Origin ;
    }
    if (Yscaling * YatiMIN < CENTER) {
      p1 = QPoint( int(Yscaling * YatiMIN), -int(iscaling * iMIN ) ) 
         + Origin ;
      r->drawLine( p2, p1 );
      p2 = QPoint( CENTER,-int(iscaling * iMIN) ) + Origin ;
      r->drawLine( p1, p2 );
    }
    else {
      p1 = QPoint( CENTER, int(iscaling * ( s * YMAX - autDemand)/rho ) ) 
         + Origin ;
      r->drawLine( p2, p1 );
    }
    r->setPen(Qt::black);
  }
  if ( (HINTFLAG == true) && ( iMAX > ihint ) ) {
    QString HBString; 
    HBString.sprintf( "T-G = %7.1f ", tax*Yhint-G);
    r->drawText( int(SCALE * 25), int(SCALE * 20), HBString );
    r->setPen(Qt::lightGray);
    p2 = - QPoint( int(Yscaling * IGhint), int(iscaling * ihint)) + Origin;
    if ( InViewOfSavings==true ) {
      if (Yhint<YMAX) {
	p3 = Yscaling * QPoint( int(Yhint),int(IGhint) ) + Origin;
	r->drawRect(QRect( p2, p3 ));
      }
      else {
	p1 = QPoint( CENTER,-int(iscaling * ihint) ) + Origin ;
	r->drawLine( p1, p2 ); 
	p3 = QPoint(-int(Yscaling * IGhint), int(Yscaling * IGhint ) ) 
           + Origin ;
	r->drawLine( p2, p3 ); 
	p1 = QPoint( CENTER, int(Yscaling * IGhint) ) 
           + Origin ;
	r->drawLine( p3, p1 ); 
      }
    }
    else {  
      p1 = Yscaling * QPoint(0,int(IGhint)) + Origin ;
      p3 = QPoint( Yscaling * IGhint * QPoint(-1, 1) + Origin );
      r->drawLine( p1, p3 );
      r->drawLine( p3, p2 ); 
      p1 = QPoint( int(Yscaling * Yhint), int(-iscaling * ihint)) + Origin ;
      r->drawLine( p2, p1 ); 
      p3 = Yscaling * Yhint * QPoint( 1, 1) + Origin ;
      r->drawLine( p1, p3 ); 
    }
  }
  r->setPen(Qt::black);
}
void CommodityMarket::UpdateISMarket(double eNew,
				     double PNew) 
{
  rho=IatiMIN/(iMAX-iMIN);
  I0=IatiMIN+rho*iMIN;
  ERate = eNew;
  P=PNew;
  update();
}
void CommodityMarket::HintLines( bool OnOffNew ) 
{
  HINTFLAG=OnOffNew;
  update();
}
void CommodityMarket::Resolution( bool LowResolution ) 
{
  SCALE = (LowResolution == true) ? 0.75 : 1. ;
  CENTER = int(SCALE * 160) ;
  Origin  = QPoint( CENTER, CENTER );
  Yscaling = SCALE * 150. / 2000. ;
  iscaling = SCALE * 150. / .3 ;
  update();
}
void CommodityMarket::SavingsConsumption( bool OnOffNew ) 
{
  InViewOfSavings=OnOffNew;
  update();
}
void CommodityMarket::OpenClosedEconomy( bool OnOffNew ) 
{
  OpenEconomy = OnOffNew;
  update();
}
void CommodityMarket::mousePressEvent( QMouseEvent *e ) 
{
  QString HBString, tmpX, tmpY;
  QPainter paint( &HBpixmap );
  int tmp ;
  double tmp1 ;
  HBpixmap.fill(QColor(255,255,240,255));
  update();                               // erase old points
  if ( e->button() == Qt::LeftButton ) {
    HBString.sprintf( " Mausposition = (%d,%d), ", e->x(),e->y() );
    if ( e->x() > CENTER  )
      tmpX.sprintf( " Y = %7.1f, ", ( e->x() - CENTER ) / Yscaling );
    else
      tmpX.sprintf( " I+G = %7.1f, ", ( CENTER - e->x() ) / Yscaling );
    if ( e->y() > CENTER ) {
      tmp1 = double(( e->y() - CENTER ) / Yscaling) ;
      if ( InViewOfSavings == true ) {
	if ( OpenEconomy == true ) 
	  tmpY.sprintf( " S+T+Im-Ex = %7.1f", tmp1 ); 
	else 
	  tmpY.sprintf( " S+T = %7.1f", tmp1 ); 
      }
      else {
	if ( OpenEconomy == true ) 
	  tmpY.sprintf( " C+I+G+Ex-Im = %7.1f", tmp1 ); 
	else 
	  tmpY.sprintf( " C+I+G = %7.1f", tmp1 ); 
      }
    }
    else {
      tmpY.sprintf( " i = %7.2f%%", ( CENTER - e->y() ) / iscaling );
    }
    HBString += tmpX ;
    HBString += tmpY ;
    if ( (e->x() > CENTER) && (e->y() < CENTER) ) {
      QPoint p1, p2;
      double i_tmp = ( CENTER - e->y() ) / iscaling ;
      double Y_tmp = ( e->x() - CENTER ) / Yscaling;
      if (OpenEconomy == false) {
	paint.setPen(Qt::lightGray);
	p1 = QPoint(e->x(), 0);
	p2 = QPoint(e->x(), int(SCALE * 320));
	paint.drawLine( p1, p2 );
	if ( S(Y_tmp)>0 ) {
	  tmp = int(Yscaling * ( I(i_tmp)+G )) ;
	  if ( InViewOfSavings==true ) {
	    p1 = QPoint( CENTER - tmp, e->y() );
	    p2 = QPoint( e->x(), CENTER + tmp ) ;
	    paint.drawRect(QRect( p1, p2 ));
	    p1 = QPoint( int(SCALE * 320), CENTER + tmp );
	    paint.drawLine(p2,p1);
	    paint.setPen(Qt::red);
#if __GNUG__ /* GNU C++ Compiler Linux */
	    paint.setFont( QFont( "helvetica", (SCALE==0.75) ? 10 : 12, 25 ) ); 
#else        /* C++ for Mickysoft Windows */
	    paint.setFont( QFont( "arial", (SCALE==0.75) ? 8 : 11, 25 ) ); 
#endif 
	    if ( S(Y_tmp) > I(i_tmp)+G ) 
	      paint.drawText( CENTER+int(SCALE * 5), int(SCALE * 290), 
                              tr("deflGap") );
	    else 
	      paint.drawText( CENTER+int(SCALE * 5), int(SCALE * 290), 
                              tr("inflGap") );
	    p1 = QPoint( e->x(), CENTER+int(Yscaling * S(Y_tmp)));
	    paint.drawLine( p1, p2 );
	  }
	  else {  
	    p1 = QPoint( e->x(), e->y() );
	    p2 = QPoint( CENTER - tmp, e->y() );
	    paint.drawLine( p1, p2 );
	    p1 = QPoint( CENTER - tmp, CENTER + tmp ) ;
	    paint.drawLine( p2, p1 );
	    p2 = QPoint( CENTER,       CENTER + tmp );
	    paint.drawLine( p1, p2 );
	    p1 = Yscaling * QPoint( 0,          int(Caut    +I(i_tmp)+G) ) 
               + Origin ;
	    p2 = Yscaling * QPoint( int(YRand), int(C(YRand)+I(i_tmp)+G) ) 
               + Origin ;
	    paint.drawLine( p1, p2 );
	    paint.setPen(Qt::red);
#if __GNUG__ /* GNU C++ Compiler Linux */
	    paint.setFont( QFont( "helvetica", (SCALE==0.75) ? 10 : 12, 25 ) ); 
#else        /* C++ for Mickysoft Windows */
	    paint.setFont( QFont( "arial", (SCALE==0.75) ? 8 : 11, 25 ) ); 
#endif 
	    if ( S(Y_tmp) > I(i_tmp)+G ) 
	      paint.drawText( CENTER+int(SCALE * 5), int(SCALE * 290), 
                              tr("deflGap") );
	    else 
	      paint.drawText( CENTER+int(SCALE * 5), int(SCALE * 290), 
                              tr("inflGap") );
	    paint.drawLine( e->x(), e->x(), 
                            e->x(), CENTER +int( Yscaling * C(Y_tmp) + tmp) );
	  }
	}
	paint.setPen(Qt::black);
      }
    }
    update();
    emit explain( HBString );
  }
  if ( e->button() == Qt::RightButton ) {
     HBpixmap.fill(QColor(255,255,240,255));
     update();
  }
}
void CommodityMarket::resizeEvent(QResizeEvent *e )
{
  QPixmap new_pixmap(e->size());
  new_pixmap.fill(QColor(255,255,240,255));
  QPainter painter(&new_pixmap);
  painter.drawPixmap(QPoint(0, 0), HBpixmap);
  HBpixmap = new_pixmap;
}
void CommodityMarket::mouseMoveEvent( QMouseEvent *e ) 
{
  QString HBString, tmp;
  double temp1;
  if (Qt::RightButton) {
    if ( (e->x() < CENTER) && (e->y() < CENTER) ) {
      HBString.sprintf(" A parameter has changed: ");
      temp1= ( CENTER - e->y() ) / iscaling ;  
      iMAX = (temp1<iMIN) ? iMIN+.0000000001 : temp1 ;
      rho  = IatiMIN / (iMAX-iMIN);
      I0   = IatiMIN + rho * iMIN;
      tmp.sprintf(" i_Max = %7.2f%%", iMAX * 100.);
      HBString += tmp ;
      emit explain( HBString );
      update();
    }
  }
  if (Qt::LeftButton) {
    if ( (e->x() < CENTER ) && (e->y() < CENTER) ) {
      HBString.sprintf(" A parameter has changed: ");
      temp1 = ( CENTER - e->x() ) / Yscaling - G - Iaut ;   
      IatiMIN = (temp1<0) ? 0. : temp1 ;
      rho  = IatiMIN / (iMAX-iMIN);
      I0   = IatiMIN + rho * iMIN;
      tmp.sprintf(" I(iMin) = %7.2f,  I0 = %7.2f", IatiMIN, I0);
      HBString += tmp ;
      emit explain( HBString );
      update();
    }
  }
}
// BoPA = Balance of Payments Account
double CommodityMarket::BoPA(double Y1,
			     double Y2,
			     double P1,
			     double P2,
			     double w) {
  double PY1 = P1 * Y1 ;
  double PY2 = w * P2 * Y2 ;
  double EXport = -( (PY1*(be1*ce1 + ae1*de1)/(de1 + be1*Y1) 
		     - PY2*(be2*ce2 + ae2*de2)/(de2 + be2*Y2) )
		  /( PY1*be1*de1/(de1 + be1*Y1) + PY2*be2*de2/(de2 + be2*Y2)));
  double pExport = PY1 *(be1*ce1 + ae1*de1 + be1*de1*EXport)/(de1 + be1*Y1);
  double exportValue = pExport * EXport;

  double import = (   PY1*(bi1*ci1 + ai1*di1)/(di1 + bi1*Y1) 
		    - PY2*(bi2*ci2 + ai2*di2)/(di2 + bi2*Y2) )
		/ (PY1*bi1*di1/(di1 + bi1*Y1)+PY2*bi2*di2/(di2 + bi2*Y2));
  double pImport = PY1 * (bi1*ci1 + ai1*di1 - bi1*di1*import)/(di1 + bi1*Y1);
  double importValue = pImport * import;
  return exportValue - importValue;
}
/*
 *  investment as a function of the interest rate
 */
double CommodityMarket::I( double iRate ) {
  if (iRate > iMAX) return 0 ;
  if (iRate < iMIN) return 9999999. ;
  return I0 + Iaut - rho * iRate ; 
}
/*
 *  consumption as a function of income retained
 */
double CommodityMarket::C( double YY ) {
  return Caut + c * (1-tax) * YY ; 
}
/*
 *  saving as a function of income retained
 */
double CommodityMarket::S( double YY ) {
  return YY - Caut - c * (1-tax) * YY ; 
}
double CommodityMarket::Y (double iRate,
			   double Y2,
			   double P1,
			   double P2,
			   double ERate,
			   double temp_s,
			   double temp_autDem,
			   double temp_rho) {
  double y0=0.,y1=0.,y2=0.,f0,f1,f2,CAB0,CAB1,CAB2;
  bool REPEAT = true ;
  CAB0 = BoPA(y0,Y2,P1,P2,ERate);
  f0 =temp_s * y0 - CAB0 - temp_autDem + temp_rho * iRate ;
  y1 = 2150. ;
  CAB1 = BoPA(y1,Y2,P1,P2,ERate);
  f1 =temp_s * y1 - CAB1 - temp_autDem + temp_rho * iRate ;
  if (f0 * f1 < 0.) {
    while ( REPEAT == true ) {
      y2 = (y0 * f1 - y1 * f0) / (f1 - f0);
      CAB2 = BoPA(y2,Y2,P1,P2,ERate);
      f2 =temp_s * y2 - CAB2 - temp_autDem + temp_rho * iRate ;
      if ( fabs(f2) < 0.2 ) {
	return y2 ; 
	REPEAT = false ;
      } 
      else {
	if (f2 * f0 < 0. ) 
	  { y1 = y2; f1 = f2; }
	else 
	  { y0 = y2; f0 = f2; }
      }
    };
  }
  return 2300. ; // omit warning
}
void CommodityMarket::ZEICHNEN ( QPainter *r, 
				 QPolygon Z, 
				 QColor color) { 
  r->setPen(color);
  Z.translate ( CENTER, CENTER );
  r->drawPolyline( Z ); 
  r->setPen(Qt::black);
}
void CommodityMarket::InitialValues( int KeynesCase ) {
  ae1=0.0080854; 
  ce1=0.07887;
  ae2=0.0153333; 
  ce2=-5.14365;
  Z=0.;
  c_h     = 7000;
  iMAX    = 0.22;  
  iMIN    = 0.01;
  IatiMIN = 1600.;
  switch (KeynesCase) {
  case 0: // liquidity trap
    if ( OpenEconomy == false ) 
         IatiMIN = 600.;
    else IatiMIN = 510.;
    t_h  = 2000; 
    G_h  =  250; 
    C0_h = 1000;
    IatiMIN = 560.;
    break ;
  case 1: // investment trap       
    t_h  = 2000; 
    G_h  =  250;
    C0_h = 1000; 
    iMAX = 0.05;
    break ; 
  case 2: // wage rate rigidity
    t_h  = 4000; 
    G_h  =  500; 
    C0_h = 1300; 
    IatiMIN = 550.;
    break ;
  case 3: // usual case
    t_h  = 4000; 
    G_h  =  500; 
    C0_h = 1300; 
    break ;
  }
  rho  = IatiMIN / (iMAX-iMIN);
  I0   = IatiMIN + rho * iMIN;
  Iaut = 0. ;
  autDemand = C0_h / 10. + I0 + Iaut + G_h / 10. ;
  set_G(10) ;       // LCD reagiert sonst nicht ???!!!
  set_C0(10) ;      // LCD reagiert sonst nicht ???!!!
  set_c(10) ;       // LCD reagiert sonst nicht ???!!!
  set_t(10) ;       // LCD reagiert sonst nicht ???!!!
  set_G(G_h) ;
  set_C0(C0_h) ;
  set_c(c_h) ;
  set_t(t_h) ;
  update();
}
void CommodityMarket::GrowthValues() 
{
  Caut = 99.238024;
  Iaut = 0. ;
  iMIN = 0.01;
  iMAX = 0.22;
  IatiMIN = 1100.;
  rho  = IatiMIN / (iMAX-iMIN);
  I0   = IatiMIN + rho * iMIN;
  autDemand = Caut + I0 + G  + Iaut;
}
void CommodityMarket::IautChanged( double C0New,
				   double GNew,
				   double INew  ) 
{
  set_C0( int (C0New * 10. ) );
  set_G( int (GNew * 10. ) );
  Iaut = INew - I0 ;
  update();
}
void CommodityMarket::EqValuesChanged( double iNew, double YNew ) 
{
  if ( (ihint!=iNew) || (Yhint!=YNew) ) { // DO NOT REPAINT if ihint = iNew !
    ihint = iNew ;
    Yhint = YNew ;
    update();
  }
}
void CommodityMarket::set_c( int tmp ) 
{
  RangeOf(tmp,0,10000);
  if ( c == tmp/10000. ) return; // don't play ping-pong with LCD
  c = tmp/10000.;
  emit cChanged( tmp );          // signal to LCD
  update();
}
void CommodityMarket::set_t( int tmp ) 
{
  RangeOf(tmp,0,10000);
  if ( tax == tmp/10000. ) return; // don't play ping-pong with LCD
  tax = tmp/10000.;
  emit tChanged( tmp );            // signal to LCD
  update();
}
void CommodityMarket::set_C0( int tmp ) 
{
  RangeOf(tmp,0,10000);
  if ( Caut == tmp/10. ) return; // don't play ping-pong with LCD
  Caut = tmp / 10. ;
  emit C0Changed( tmp );         // signal to LCD
  update();
}
void CommodityMarket::set_G( int tmp ) 
{
  RangeOf(tmp,0,10000);
  if ( G == tmp/10. )  return;   // don't play ping-pong with LCD
  G = tmp / 10. ;
  emit GChanged( tmp );          // signal to LCD
  update();
}
void CommodityMarket::AdjustmentEx() 
{
  if (OpenEconomy==true) {
    QString HBString;
    HBString.sprintf("Running ...");
    emit explain( HBString );
    Z = - ( BoPA(Yhint,4500.,P,2.,ERate) + 3000. * (ihint - 0.09) ) ;
    if (fabs(Z)>0.5) {
      if (Z>0) Z = 10.;
      else     Z =-10.;
    }
    fprintf(stderr,"DEx = BoPA = %8.3f,   NCI=%f\n", 
	    BoPA(Yhint,4500.,P,2.,ERate),3000. * (ihint - 0.09));
    DELTA1 = -Z * (de1 * be1) / (de1 + be1) ;
    DELTA2 =  Z * (de2 * be2) / (de2 + be2) ;
    ae1 = ae1 + DELTA1 ;
    ce1 = ce1 + DELTA1 ;
    ae2 = ae2 + DELTA2 ;
    ce2 = ce2 + DELTA2 ; 
    update();
    HBString.sprintf("Finished ...");
    emit explain( HBString );
  }
}
void CommodityMarket::ReadParam( double Param[8] ) 
{
  Param[0] = ae1;
  Param[1] = ce1; 
  Param[2] = ae2;
  Param[3] = ce2; 
  return ;
}
