#include "LCD.h"
#include <QtWidgets/QLCDNumber>  
#include <QtWidgets/QScrollBar>

LCDRange::LCDRange( QWidget *parent )
  : QWidget( parent )
{
  init();
}
LCDRange::LCDRange( const char *s, QWidget *parent )
        : QWidget( parent ) 
{
  init();
  setText( s );
}
void LCDRange::init() {
  lcd  = new QLCDNumber( 6, this );
  lcd->setSegmentStyle( QLCDNumber::Flat );
  lcd->setMode( QLCDNumber::Dec );
  lcd->setSmallDecimalPoint(true);                     
  sBar = new QScrollBar(Qt::Horizontal, this);
  label = new QLabel( this );
  connect( sBar, SIGNAL(valueChanged(int)), lcd, SLOT(display(int)) );
  connect( sBar, SIGNAL(valueChanged(int)), SIGNAL(NewVal(int)) );
}
int LCDRange::value() const {
  return sBar->value();
}
/*
const char *LCDRange::text() const {
  return label->text();
  //return "dummy";
}
*/
void LCDRange::setVValue( int value )     // domain 0<x<10000
{
  sBar->setValue( value ) ;
  lcd->display( value );
}
void LCDRange::setGValue( int value )     // domain 0<x<1000
{
  lcd->setSmallDecimalPoint(true);
  sBar->setValue( value ) ;
  lcd->display(0.1 * value);
}
void LCDRange::setPValue( int value )     // domain 0<x<10
{
  lcd->setSmallDecimalPoint(true);                     
  sBar->setValue( value ) ;
  lcd->display(.001 * value);
}
void LCDRange::setValue( int value )     // domain 0<x<1
{
  lcd->setSmallDecimalPoint(true);
  sBar->setValue( value ) ;
  lcd->display( .0001 * value );
}
void LCDRange::setRange( int minVal, int maxVal ) {
  if ( minVal < 0 || maxVal > 10000 || minVal > maxVal ) {
    /*    warning( "LCDRange::setRange(%d,%d)\n"
	     "\tRange must be 0..10000\n"
	     "\tand minVal must not be greater than maxVal",
	     minVal, maxVal );
    */
    return;
  }
  sBar->setRange( minVal, maxVal );
  sBar->setPageStep( 100 );
  sBar->setSingleStep( 50 );
}
void LCDRange::setText( const char *s ) {
  label->setText( s );
  label->setIndent(3);
  label->setFrameStyle(QFrame::Panel | QFrame::Sunken);
}
void LCDRange::resizeEvent( QResizeEvent * ) {
  lcd->resize( width(), height() - 41 - 5 );
  sBar->setGeometry( 0, lcd->height() + 5, width(), 16 );
  label->setGeometry( 0, lcd->height() + 16 + 5, width(), 20 );
}
