/*
 * graphical representation of the 
 * MONEY MARKET
 * initial situation and after changes of parameters (updates)
 */
#define RangeOf(x,a,b)  if ( x < a ) x = a; if ( x > b )  x = b
#include "LMMarket.h"
#include <QPainter>
#include <QPolygon>
#include <QRect>
#include "IS-LM.h"

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

MoneyMarket::MoneyMarket( QWidget *parent )
        : QWidget( parent ) 
{
  Resolution( true ); // must be first
  HINTFLAG = true;
  k = 0.07;           // Cambridge k = 1/V
  M = 110.;           // nominal money supply
  ihint = 0.151;
  YMAX = 2000. * 160./150.;
  Yhint = 1219.8;
  InitialValues( 3 );
}
void MoneyMarket::paintEvent( QPaintEvent * ) 
{
  QPainter r( this ); 
  DrawAxes( &r );
  paintMoneyMarket( &r ) ; 
  if (P==0) return;
  emit LMMarket2ISLM( k, M, P, delta, LS0, iMAX ); 
}
void MoneyMarket::DrawAxes( QPainter *r )
{
  r->setBrush(QBrush("#fffff0"));  
  r->setPen(Qt::black);
  r->drawRect( 0, 0, 1.999 * CENTER, 1.999 * CENTER );
  r->setBrush(Qt::NoBrush); 
  r->drawLine( int(SCALE * 10), CENTER, int(SCALE * 310), CENTER );
  r->drawLine( CENTER, int(SCALE * 10), CENTER, int(SCALE * 310) );
#if __GNUG__ /* GNU C++ Compiler Linux */
  r->setFont( QFont( "helvetica", (SCALE==0.75) ? 10 : 12, 25 ) ); 
#else        /* C++ for Mickysoft Windows */
  r->setFont( QFont( "arial", (SCALE==0.75) ? 8 : 11, 25 ) ); 
#endif 
  r->drawText( int(SCALE * 165), int(SCALE * 15),  "i" );
  r->drawText( int(SCALE * 300), int(SCALE * 155), "Y" );
  r->drawText( int(SCALE * 165), int(SCALE * 310), "L" );
  r->drawText( int(SCALE * 172), int(SCALE * 314), "T" );
  r->drawText( int(SCALE *   5), int(SCALE * 145), "L" );
  r->drawText( int(SCALE *  12), int(SCALE * 149), "S" );
  r->setPen( QColor(34, 139,  34) );
  r->drawText( int(SCALE * 200), int(SCALE * 15), tr("LM curve") );
  r->setPen(Qt::black);
}
void MoneyMarket::paintMoneyMarket( QPainter *r ) 
{
  QPoint p1, p2;
  /*
   *   LT curve
   */
  if (SCALE * k*YMAX < 161 )
    p1 = QPoint( int(Yscaling*YMAX), int(SCALE*k*YMAX) ) + Origin ;
  else 
    p1 = QPoint( int(Yscaling*(160./k)), CENTER ) + Origin ;
  r->drawLine( Origin, p1 );
  /*
   *   LS curve
   */
  p1 = QPoint( 0, CENTER -int(iscaling * iMIN) );
  p2 =-QPoint( int(SCALE * LS_kink), int(iscaling * iMIN) ) + Origin ;
  r->drawLine( p1, p2 );
  p1 = QPoint( 0,-int(iscaling * iMAX)) + Origin ;
  r->drawLine( p2, p1 );
  p2 = QPoint( CENTER, 0 );
  r->drawLine( p1, p2 );
  /*
   *  money demand = money supply
   */
  if (M/P<=160) {
    p1 = QPoint( 0, int(SCALE * M/P) ) + Origin ;
    p2 = QPoint(-int(SCALE * M/P), 0 ) + Origin ;
    r->drawLine( p1, p2 );
  }
  else {
    p1 = QPoint(-int(SCALE * (M/P-160)), CENTER ) + Origin ;
    p2 = QPoint(-CENTER, int(SCALE * (M/P-160)) ) + Origin ;
    r->drawLine( p1, p2 );
  }
  r->setPen(QColor(34, 139,  34));
  if (M/P<LS_kink) { 
    p1 = QPoint(0, int(iscaling * (M/P-LS0)/delta) ) + Origin ; 
  }
  else { 
    p2 = QPoint( 0, -int(iscaling * iMIN) ) + Origin ;
    if (Yscaling * (M/P-LS_kink)/k <= CENTER)
      p1 = QPoint( int(Yscaling * (M/P-LS_kink)/k),
                  -int(iscaling * iMIN)) 
         + Origin ;
    else 
      p1 = QPoint(CENTER,-int(iscaling * iMIN)) + Origin ;
    r->drawLine( p2, p1 );
  }
  if (Yscaling * (M/P)/k <= CENTER ) {
    p2 = QPoint( int(Yscaling * (M/P)/k), -int(iscaling * iMAX) ) + Origin ; 
    r->drawLine( p1, p2 );
    p1 = QPoint( int(Yscaling * (M/P)/k), -CENTER ) + Origin ; 
    r->drawLine( p2, p1 );
  }
  else {
    p2 = QPoint( CENTER, -int(iscaling * (M/P - LS0 - k * YMAX)/(-delta))) 
       + Origin;
    r->drawLine( p1, p2 );
  }
  r->setPen(Qt::black);
  /*
   *  Hint lines
   */
  if (HINTFLAG == true)  {
    r->setPen(Qt::lightGray);
    if ( ( ihint < iMAX ) && (ihint > iMIN ) ) {
      if ( Yhint<=YMAX ) {
	p1 =-QPoint( int(SCALE * (LS0 - delta * ihint)), int(iscaling * ihint)) 
           + Origin;
	p2 = QPoint( int(Yscaling * Yhint), int(SCALE * k * Yhint) ) 
           + Origin;
	r->drawRect( QRect( p1, p2 ) );
      }
      else {
	p1 = QPoint( CENTER, -int(iscaling * ihint)) + Origin ;
	p2 = QPoint( -int(SCALE * (LS0 - delta * ihint)), 
                     -int( iscaling * ihint) ) 
           + Origin ;
	r->drawLine( p1, p2 );
	p1 = QPoint( -int(SCALE * (LS0 - delta * ihint)), 
                     int( SCALE * k * Yhint) ) 
           + Origin ;
	r->drawLine( p2, p1 );
	p2 = QPoint(CENTER,int( SCALE * k * Yhint)) + Origin ;
	r->drawLine( p1, p2 );
      }
    }
  }
  r->setPen(Qt::black);
}
void MoneyMarket::UpdateLMMarket( double PNew ) 
{
  P = PNew;
  update();
}
void MoneyMarket::HintLines( bool OnOffNew ) 
{
  HINTFLAG=OnOffNew;
  update();
}
void MoneyMarket::Resolution( bool LowResolution ) 
{
  SCALE = (LowResolution == true) ? 0.75 : 1. ;
  CENTER = int(SCALE * 160) ;
  Origin  = QPoint( CENTER, CENTER );
  Yscaling = SCALE * 150. / 2000. ;
  iscaling = SCALE * 150. / .3 ;
  update();
}
void MoneyMarket::mousePressEvent( QMouseEvent *e ) {
  QString HBString, tmpX, tmpY;
  if ( ( e->button() == Qt::LeftButton ) || ( e->button() == Qt::RightButton ) ) {
    HBString.sprintf( " Mausposition = (%d,%d), ", e->x(),e->y() );
    if ( e->x() > CENTER ) 
      tmpX.sprintf( " Y = %7.1f, ", ( e->x() - CENTER ) / Yscaling );
    else 
      tmpX.sprintf( " LS(i) = %7.1f, ", ( CENTER - e->x() ) / SCALE );
    if ( e->y() > CENTER ) 
      tmpY.sprintf( " LT(Y) = %7.1f", ( e->y() - CENTER ) / SCALE );
    else 
      tmpY.sprintf( " i = %7.2f%%", ( CENTER - e->y() ) / iscaling );
    HBString += tmpX ;
    HBString += tmpY ;
    emit explain( HBString );
  }
}
void MoneyMarket::mouseMoveEvent( QMouseEvent * e) 
{
  QString HBString, tmp;
  if (Qt::RightButton) {
    HBString.sprintf(" A parameter has changed: ");
    if ( (e->x() > CENTER) && (e->y() > CENTER) ) {
      k = (( e->y() - CENTER ) / SCALE) 
	/ (( e->x() - CENTER ) / Yscaling ) ;
      tmp.sprintf(" k = %7.5f", k);
      set_k( int(k * 10000.) );
    }
    if ( (e->x() < CENTER) && (e->y() > CENTER) ) {
      M = ( e->y() - e->x() ) * P / SCALE ;
      tmp.sprintf(" M = %7.1f", M );
      set_M( int(M * 10.) );
      update();
    }
    if ( (e->x() < CENTER) && (e->y() < CENTER) ) {
      double temp1 = ( CENTER - e->y() ) / iscaling ;
      if (temp1<=iMIN) 
	   iMAX = iMIN+.000000001 ;      
      else iMAX = temp1 ;      
      tmp.sprintf(" i_Max = %7.2f%%", iMAX * 100. );
      LS0  = LS_kink + delta * iMIN;
      delta= LS_kink /(iMAX-iMIN);
      update();
      emit LMMarket2ISLM( k, M, P, delta, LS0, iMAX);
    }
  }
  if (Qt::LeftButton) {
    if ( (e->x() < CENTER) && (e->y() < CENTER) ) {
      LS_kink = ( CENTER - e->x() ) / SCALE ;      
      tmp.sprintf(" LS_0 = %7.2f", LS_kink);
      delta= LS_kink /(iMAX-iMIN);
      LS0  = LS_kink + delta * iMIN;
      update();
    }
  }
  HBString += tmp ;
  emit explain( HBString );
}
void MoneyMarket::InitialValues( int KeynesCase ) 
{
  iMAX = 0.20;
  iMIN = 0.02; 
  LS_kink = 90.;
  P = 1. ;
  delta= LS_kink /(iMAX-iMIN);
  LS0  = LS_kink + delta * iMIN;
  k_h =  700; 
  switch (KeynesCase) {
  case 0: // liquidity trap
    M_h = 2100; 
    break ;
  case 1: // investment trap       
    M_h = 1200;  
    P = 2.1 ;
    break ; 
  case 2: // wage rate rigidity
    M_h = 1100; 
    break ;
  case 3: // usual case
    M_h = 1100; 
    break ;
  }
  set_M(10);   // LCD reagiert sonst nicht !?
  set_k(10);   // LCD reagiert sonst nicht !?
  set_M(M_h);
  set_k(k_h);
  update();
}
void MoneyMarket::EqValuesChanged( double iNew, double YNew ) 
{
  if ( (ihint!=iNew) || (Yhint!=YNew) ) { // DO NOT REPAINT if ihint = iNew !
    ihint = iNew ;
    Yhint = YNew ;
    update();
  }
}
void MoneyMarket::set_k( int tmp ) 
{
  RangeOf(tmp,2,10000);
  if ( k == tmp/10000. ) return; // don't play ping-pong with LCD
  k = tmp / 10000. ;
  emit kChanged( tmp );          // signal to LCD
  update();
}
void MoneyMarket::set_M( int tmp ) 
{
  RangeOf(tmp,0,10000);
  if ( M == tmp/10. ) return;    // don't play ping-pong with LCD
  M = tmp / 10. ;
  emit MChanged( tmp );          // signal to LCD
  update();
}
